/*
Copyright (C) 2014 NTT DATA Corporation

This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License
as published by the Free Software Foundation, version 2.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
 */
package com.clustercontrol.cloud.cloudn;

import java.text.MessageFormat;
import java.util.MissingResourceException;

import com.clustercontrol.cloud.CloudManagerFault;

public enum ErrorCode {
	UNEXPECTED("unexpected"),

	SERVICE_OFFERING_NOT_FONUD("message.validation.cloudn.service_offering.not_found"),
	VM_NOT_FONUD("message.validation.cloudn.vm.not_found"),
	DISK_OFFERING_NOT_FONUD("message.validation.cloudn.disk_offering.not_found"),
	ZONE_NOT_FONUD("message.validation.cloudn.zone.not_found"),
	INSTANCE_CANNOT_DELETE_WHILE_LAUNCHING("message.validation.cloudn.instance.cannot_delete_while_launching"),
	STORAGE_NOT_FONUD("message.validation.cloudn.storage.not_found"),
	STORAGE_CANNOT_BACKUP_ROOT_VOLUME("message.validation.cloudn.storage.cannot_backup_root_volume"),
	STORAGE_CANNOT_CREATE_FROM_SNAPSHOT_OF_ROOT_VOLUME("message.validation.cloudn.storage.cannot_create_from_snapshot_of_root_volume"),
	IMAGE_INVALID_ID("message.validation.cloudn.image.invalid_id"),
	FILTER_INVALID("message.validation.cloudn.filter.invalid"),
	INSTANCE_BACKUP_NEED_VM_TO_BE_STOPPED("message.validation.cloudn.instance_backup.need_vm_to_stopped"),
	CLOUDUSER_MUST_USE_SAME_PASSWORD_OF_ACCOUNT("message.validation.cloudn.clouduser.must_use_same_password_of_account"),
	JOB_ID_UNREGISTERED("message.validation.cloudn.job_id_unregistered"),
	;
	
	private String messageId;

	private ErrorCode(String messageId) {
		this.messageId = messageId;
	}
	
	public String getMessage(Object... args) {
		MessageFormat messageFormat = new MessageFormat(getMessage());
		return messageFormat.format(args);
	}

	public String getMessage() {
		try {
			return Messages.getString(messageId);
		}
		catch (MissingResourceException e) {
			return messageId;
		}
	}

	public CloudManagerFault cloudManagerFault(Object... args) {
		return new CloudManagerFault(getMessage(args), this.name());
	}
}
